package net.geraldloeffler.tasktracker2006.service;

import java.util.Date;
import java.util.List;
import javax.ejb.EJB;
import javax.ejb.Stateless;
import javax.jws.WebService;

import net.geraldloeffler.tasktracker2006.backend.accounting.AccountingSystemClient;
import net.geraldloeffler.tasktracker2006.dao.EntryDAO;
import net.geraldloeffler.tasktracker2006.domain.Entry;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * Webservice implementation of Service layer operations relating to Entry domain objects.
 * 
 * @author Gerald Loeffler
 */
@Stateless
@WebService
public class EntryServiceBean implements EntryService {
  private static final Log log = LogFactory.getLog(EntryServiceBean.class);
  
  @EJB
  private EntryDAO               entryDAO;
  @EJB
  private AccountingSystemClient accountingSystem;
  @EJB
  private SecurityRules          securityRules;

  public Entry createEntry(Entry entry) {
    if (entry == null) return null;

    securityRules.checkCreateEntry(entry);
    
    entry = entryDAO.merge(entry); 
    replicateToAccountingSystem(entry);

    return entry;
  }

  public Entry updateEntry(Entry entry) {
    if (entry == null) return null;
    
    securityRules.checkUpdateEntry(entry);

    entry = entryDAO.merge(entry); 
    replicateToAccountingSystem(entry);

    return entry;
  }

  private void replicateToAccountingSystem(Entry entry) {
    if (entry == null) return;

    // only replicate if both from and to date/time are set
    if (entry.getFrom() == null || entry.getTo() == null) {
      log.debug("replicateToAccountingSystem(): not replicating entry because from/to are not both set yet");
    } else {
      log.debug("replicateToAccountingSystem(): replicating entry to accounting system");
      accountingSystem.replicateEntryToAccountingSystem(entry);
    }
  }
  
  public Entry getEntryByID(int id) {
    return entryDAO.findByID(id);
  }

  public List<Entry> getEntriesInPeriod(Date from, Date to) {
    if (from == null && to == null) return entryDAO.findAll();
    if (from == null)               return entryDAO.findActiveAtOrBefore(to);
    if (to == null)                 return entryDAO.findActiveAtOrAfter(from);
    return entryDAO.findActiveInPeriod(from, to);
  }
}
